SystemOrganization addCategory: #'Dominion-Cards'!
SystemOrganization addCategory: #Dominion!
SystemOrganization addCategory: #'Dominion-Test'!

TestCase subclass: #CoreTest
	instanceVariableNames: 'board deck1 deck2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!CoreTest methodsFor: 'private' stamp: 'MichaelOSullivan 7/10/2012 21:55'!
setupGame
	board := Board instance.
	board initialize.
	board decks: 2.
	deck1 := board deck: 1.
	deck2 := board deck: 2.
	board startGame.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 14:17'!
testActionCommand
	self setupGame.
	
	"Send an action command to the wrong deck.  Make sure it returns an error."
	self should: [(ActionCommand deck: 2 card: Smithy) do.] raise: Error.
	
	"Send an action command on a non action card.  Make sure it returns an error."
	self should: [(ActionCommand deck: 1 card: Copper) do.] raise: Error.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	deck1 z_addCardToHand: Smithy.
	(ActionCommand deck: 1 card: Smithy) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	"Send an action command after the action phase is complete.  Make sure it returns an error."
	deck1 z_addCardToHand: Smithy.
	self should: [(ActionCommand deck: 1 card: Smithy) do.] raise: Error.
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 23:19'!
testAdventurer
	|treasure|
	self setupGame.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	treasure := (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size.
	deck1 z_addCardToHand: Adventurer.
	(ActionCommand deck: 1 card: Adventurer) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size = (treasure + 2).
	self assert: (deck1 play at: 1) class = Adventurer.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 23:48'!
testBureaucrat
	self setupGame.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Bureaucrat.
	(ActionCommand deck: 1 card: Bureaucrat) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw last class = Silver.
	self assert: (((deck2 hand size = 4) and: (deck2 z_draw last class = Estate)) or: ((deck2 hand select: [:each | each class = Copper]) size = 5)).
	self assert: (deck1 play at: 1) class = Bureaucrat.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 14:17'!
testBuyCommand
	self setupGame.
	
	"Make sure the phase of each deck is correct.  Deck 1 should be at Action since that is the first phase of a turn.  Deck 2 should be at None because it is not Deck 2s turn."
	self assert: deck1 phase = #Action.
	self assert: deck2 phase = #None.
	
	"Send a buy command to the correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	(BuyCommand deck: 1 card: Copper) do.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Copper.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.
	
	"Send a buy command to the wrong deck.  Make sure it returns an error."
	self should: [(BuyCommand deck: 1 card: Copper) do.] raise: Error.
	
	"Send a buy command to the new correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	(BuyCommand deck: 2 card: Copper) do.
	self assert: deck2 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Copper.
	self assert: deck2 phase = #None.
	self assert: deck1 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/6/2012 00:40'!
testCellar
	| victory | 
	self setupGame.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	victory := (deck1 hand select: [:each | each class inheritsFrom: VictoryCard]) size.
	deck1 z_addCardToHand: Cellar.
	(ActionCommand deck: 1 card: Cellar) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = victory).
	self assert: ((1 to: victory) select: [:n | (deck1 z_discard at: n) class = Estate]) size = victory.
	self assert: (deck1 play at: 1) class = Cellar.
	self assert: deck1 phase = #Action.
	self assert: deck1 actions = 1.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 14:49'!
testEndTurnCommand
	self setupGame.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	self should: [(EndTurnCommand deck: 2) do.] raise: Error.
	
	"Send an end turn command to deck1."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	(EndTurnCommand deck: 1) do.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	self should: [(EndTurnCommand deck: 1) do.] raise: Error.
	
	"Send an end turn command to deck2."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	(EndTurnCommand deck: 2) do.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 5.
	self assert: deck2 z_draw size = 0.
	self assert: deck2 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	self should: [(EndTurnCommand deck: 2) do.] raise: Error.
	
	"Send a play treasure command to deck1."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(4) play(1)"
	(TreasureCommand deck: 1 card: Copper) do.
	
	"Send a buy command to deck1.  This also ends the turn because the deck is out of buys"
	"Before:     draw(0) discard(5) hand(4) play(1)"
	"After buy: draw(0) discard(6) hand(4) play(1)"
	"After end: draw(6) discard(0) hand(5) play(0)"
	(BuyCommand deck: 1 card: Copper) do.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 0.
	self assert: deck1 z_draw size = 6.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	self should: [(EndTurnCommand deck: 1) do.] raise: Error.
	
	"Send an end turn command to deck2."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:   draw(5) discard(0) hand(5) play(0)"
	(EndTurnCommand deck: 2) do.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 0.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 phase = #None.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 16:50'!
testGameEnd
	self testSetup.
	
	1 to: 7 do: [:a |
		1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
		1 to: 3 do: [:b |(TreasureCommand deck: 1 card: Gold) do].
		(BuyCommand deck: 1 card: Province) do.
		
		(EndTurnCommand deck: 2) do.
		
		self assert: board gameOver not.
		self should: [deck1 victoryPoints] raise: Error.
	].
	
	1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
	1 to: 3 do: [:b |(TreasureCommand deck: 1 card: Gold) do].
	(BuyCommand deck: 1 card: Province) do.
	
	self assert: board gameOver.
	self assert: board winner = 1.
	self assert: deck1 victoryPoints = ((6*8)+3).
	self assert: deck2 victoryPoints = 3.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/6/2012 01:59'!
testMine
	| copper | 
	self setupGame.
	
	"Play a Mine.  We are using a default player who will trash copper for silver, so make sure he does so."
	copper := (deck1 hand select: [:each | each class = Copper]) size.
	deck1 z_addCardToHand: Mine.
	(ActionCommand deck: 1 card: Mine) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: Board instance trashTop class = Copper.
	self assert: (deck1 z_discard size = 0).
	self assert: (deck1 hand select: [:each | each class = Copper]) size = (copper - 1).
	self assert: (deck1 handContains: Silver).
	self assert: (deck1 play at: 1) class = Mine.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/10/2012 22:01'!
testSetup
	self setupGame.

	self assert: (deck1 z_draw select: [:each | each class = Copper]) size + (deck1 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck1 z_draw select: [:each | each class = Estate]) size + (deck1 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: (deck2 z_draw select: [:each | each class = Copper]) size + (deck2 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck2 z_draw select: [:each | each class = Estate]) size + (deck2 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: ((board cardsRemaining: Copper) = (60 - (7*2))).
	self assert: ((board cardsRemaining: Estate) = 8).! !

!CoreTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/10/2012 21:44'!
testSmithy
	self setupGame.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Smithy.	

	(ActionCommand deck: 1 card: Smithy) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 14:18'!
testTreasureCommand
	self setupGame.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	(TreasureCommand deck: 1 card: Copper) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 1.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Send a treasure command to the wrong deck.  Make sure it returns an error."
	self should: [(TreasureCommand deck: 2 card: Copper) do.] raise: Error.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	(TreasureCommand deck: 1 card: Copper) do.
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 3.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Try to buy an estate.  This costs 2, which we have due to having played 2 coppers"
	(BuyCommand deck: 1 card: Estate) do.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Estate.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/5/2012 00:14'!
testVillage
	self setupGame.
	
	"Play a Village. Make sure a card was drawn, the number of actions is now 2, and the phase is still #Action"
	deck1 z_addCardToHand: Village.
	deck1 z_addCardToHand: Smithy.
	deck1 z_addCardToHand: Smithy.
	(ActionCommand deck: 1 card: Village) do.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 2.
	self assert: (deck1 play at: 1) class = Village.
	self assert: deck1 phase = #Action.
	
	"Make sure we can now play two more cards."
	(ActionCommand deck: 1 card: Smithy) do.
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 2) class = Smithy.
	self assert: deck1 phase = #Action.
	
	(ActionCommand deck: 1 card: Smithy) do.
	self assert: deck1 play size = 3.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 3) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	! !

Object subclass: #Board
	instanceVariableNames: 'cards decks players trash currentDeck gameOver globalEvents playerEvents'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Dominion'!

!Board commentStamp: 'JohnParker 7/10/2012 19:50' prior: 0!
A Board is a singleton representing the overall game state of Dominion.

Instance Variables
	cards:			Bag
	currentDeck:	Number
	decks:			OrderedCollection
	trash: 			OrderedCollection 
	gameOver:		Boolean
	globalEvents	Array
	playerEvents	Array

cards
	- a Bag containing the card classes available and the number left of each class

currentDeck
	- the deck whose turn it currently is.

decks
	- the list of decks in the game.  

trash
	- a list of cards that have been trashed so far.

gameOver 
	- indicates whether the game is over.

globalEvents
	- the events that are sent to all players when triggered.
	
playerEvents
	- the events that are sent to a specific player when triggered.  These events carry information that is private to a given player.!

!Board class methodsFor: 'instance creation' stamp: 'JohnParker 7/3/2012 21:31'!
instance
	^UniqueInstance ifNil: [UniqueInstance := self basicNew initialize]! !

!Board class methodsFor: 'instance creation' stamp: 'JohnParker 7/3/2012 21:26'!
new
	self Error  Nothing more expected ->'Board is a singleton that is returned using the message instance'! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/3/2012 20:45'!
addCard: aCardClass num: aNumber
	cards add: aCardClass withOccurrences: aNumber! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/6/2012 01:10'!
atDeck: deckNum player: aPlayer
	|deck|
	aPlayer deck: deckNum.
	players at: deckNum put: aPlayer.
	
	globalEvents do: [:event | self when: event send: event to: aPlayer].
	
	deck := decks at: deckNum.
	"playerEvents do: [:event | deck when: event send: event to: aPlayer]."
	deck when: #resolveAction send: #resolveAction: to: aPlayer with: Card.
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 21:38'!
cardStackEmpty: aCardClass
	^(cards includes: aCardClass) not! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 02:27'!
cardsRemaining: aCardClass
	^cards occurrencesOf: aCardClass! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:56'!
currentDeck
	^decks at: currentDeck! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:29'!
deck: aNumber
	^decks at: aNumber ifAbsent: [^nil]! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/6/2012 00:41'!
decks: aNumber
	| victoryNumber deck|
	cards := Bag new.
	
	"Add the standard cards to the game"
	cards add: Copper withOccurrences: 60.
	cards add: Silver withOccurrences: 40.
	cards add: Gold withOccurrences: 30.
	
	victoryNumber := (aNumber = 2) ifTrue: [8] ifFalse: [12].
	cards add: Estate withOccurrences: (victoryNumber + (3*aNumber)).
	cards add: Duchy withOccurrences: victoryNumber.
	cards add: Province withOccurrences: victoryNumber.
	cards add: Curse withOccurrences: (10 * (aNumber - 1)).
	
	1 to: aNumber do: [:num | 
		| player |
		deck := Deck number: num.
		decks add: deck.
		player := Player new.
		players add: player.
		self atDeck: num player: player.
	].
	currentDeck := 1.
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 21:28'!
events
	|list|
	list := OrderedCollection new.
	list addAll: globalEvents.
	list addAll: playerEvents.
	^list.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/6/2012 00:28'!
execute: aCommand
	aCommand do.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
gameOver
	^gameOver.! !

!Board methodsFor: 'initialize-release' stamp: 'JohnParker 7/5/2012 23:32'!
initialize
	decks := OrderedCollection new.
	players := OrderedCollection new.
	currentDeck := 1.
	cards := Bag new.
	trash := OrderedCollection new.
	gameOver := false.
	globalEvents := {#cardPlayed. #cardDiscarded. #cardRevealed. #playerTurn. #playerShuffle. #cardTaken. #cardTrashed.}.
	playerEvents := {#cardDrawn. #cardPeeked. #resolveAction. #respondToCard. #badCommand.}.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:07'!
nextDeck
	(cards includes: Province) ifFalse: [
		gameOver := true.
		currentDeck := 0.
	] ifTrue: [
		currentDeck := (currentDeck rem: decks size) + 1.
		(decks at: currentDeck) startTurn.
	]! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/4/2012 23:43'!
otherDecks
	^((1 to: decks size) reject: [:num | num = currentDeck]) collect: [:num | decks at: num].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/4/2012 13:49'!
startGame
	(decks at: 1) startTurn.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/5/2012 20:25'!
takeCard: aCardClass deck: aNumber
	cards remove: aCardClass ifAbsent: [^nil].
	self triggerEvent: #cardTaken withArguments: {aNumber. aCardClass}.
	^aCardClass basicNew.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/5/2012 20:36'!
trashCard: aCard deck: deckNumber
	trash addLast: aCard.
	self triggerEvent: #cardTrashed withArguments: {deckNumber. aCard class}.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 01:56'!
trashTop
	^trash last.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
winner
	|tempDecks|
	gameOver ifFalse: [self error: 'The game is not over so there is no winner yet.'].
	tempDecks := OrderedCollection new.
	tempDecks addAll: decks.
	tempDecks sort: [:first :second | first victoryPoints > second victoryPoints].
	^decks indexOf: (tempDecks at: 1).! !

Object subclass: #Card
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Card commentStamp: 'JohnParker 7/3/2012 19:31' prior: 0!
A Card is an abstract class at the top of the card hierarchy.  It represents a generic card in Dominion.!

Card subclass: #ActionCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCard commentStamp: 'JohnParker 7/4/2012 14:49' prior: 0!
An ActionCard is an abstract class representing a Dominion action card.  !

!ActionCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:55'!
cardName
	^'Action'! !

!ActionCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^Color white! !

!ActionCard methodsFor: 'private' stamp: 'JohnParker 7/6/2012 01:30'!
notifyBadResolve: argArray
	Board instance currentDeck triggerEvent: #badCommand withArguments: {#resolveAction. self class. argArray.}.
	! !

!ActionCard methodsFor: 'private' stamp: 'JohnParker 7/6/2012 01:37'!
notifyBadResolve: argArray message: aString
	Board instance currentDeck triggerEvent: #badCommand withArguments: {#resolveAction. self class. argArray. aString}.
	! !

!ActionCard methodsFor: 'game' stamp: 'JohnParker 7/5/2012 22:03'!
resolve: argVector! !

ActionCard subclass: #Adventurer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:01'!
cardName
	^'Adventurer'! !

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cost
	^6! !

!Adventurer methodsFor: 'game' stamp: 'JohnParker 7/5/2012 23:50'!
playCard
	| numTreasure card deck|
	numTreasure := 0.
	deck := Board instance currentDeck.
	[numTreasure < 2] whileTrue: [
		card := deck drawCardToPeek.
		card isNil ifTrue: [numTreasure := 2].
		
		(card class inheritsFrom: TreasureCard) ifTrue: [
			deck fromPeekToHand: card class. 
			numTreasure := numTreasure + 1
		] ifFalse: [
			deck fromPeekToDiscard: card class
		].
	].
	^true.! !

ActionCard subclass: #Bureaucrat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:30'!
cardName
	^'Bureaucrat'! !

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/5/2012 23:34'!
playCard
	Board instance currentDeck gainCardToDraw: Silver.
	Board instance otherDecks do: [:each | each bureaucrat].
	^true.! !

ActionCard subclass: #Cellar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cardName
	^'Cellar'! !

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^2! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/6/2012 01:12'!
playCard
	|deck|
	deck := Board instance currentDeck.
	deck addActions: 1.
	^false
	! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/6/2012 01:20'!
resolve: cardVector
	| deck |
	deck := Board instance currentDeck.
	
	(deck handContainsAll: cardVector) ifFalse: [
		deck triggerEvent: #badCommand withArguments: {#resolveAction. self class. cardVector.}.
		^false.
	].
	
	cardVector do: [:each | deck fromHandToDiscard: each].
	
	1 to: cardVector size do: [:a | deck drawCardToHand].
	^true.! !

ActionCard subclass: #Mine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 01:23'!
cardName
	^'Cellar'! !

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 01:23'!
cost
	^2! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/6/2012 01:23'!
playCard
	^false
	! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/6/2012 02:04'!
resolve: cardVector
	| deck trashCard gainCard|
	deck := Board instance currentDeck.
	
	
	(cardVector size = 2) ifFalse: [
		self notifyBadResolve: cardVector message: 'You must select a treasure to trash and a treasure to gain.'.
		^false.
	].
	
	trashCard := cardVector first.
	gainCard := cardVector second.
	(deck handContains: trashCard) ifFalse: [
		self notifyBadResolve: cardVector message: ('You do not have a ' , trashCard cardName , 'in your hand to trash.').
		^false.
	].
	
	((trashCard inheritsFrom: TreasureCard) and: (gainCard inheritsFrom: TreasureCard)) ifFalse: [
		self notifyBadResolve: cardVector message: ('Mine only works on treasure cards.').
		^false.
	].

	((trashCard cost + 3) >= gainCard cost) ifFalse: [
		self notifyBadResolve: cardVector message: ('You can only gain a treasure costing at most 3 more than the trashed card.').
		^false.
	].
	
	deck trashCardFromHand: trashCard.
	deck gainCardToHand: gainCard.
	
	^true.! !

ActionCard subclass: #Smithy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:57'!
cardName
	^'Smithy'! !

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Smithy methodsFor: 'game' stamp: 'MichaelOSullivan 7/10/2012 21:45'!
playCard
	3 timesRepeat: [Board instance currentDeck drawCardToHand].
	^true.! !

ActionCard subclass: #Village
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
cardName
	^'Village'! !

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^3! !

!Village methodsFor: 'game' stamp: 'JohnParker 7/5/2012 23:34'!
playCard
	|deck|
	deck := Board instance currentDeck.
	deck drawCardToHand.
	deck addActions: 2.
	^true.
	! !

!Card class methodsFor: 'game' stamp: 'JohnParker 7/4/2012 13:18'!
buy
	^Board instance currentDeck gainCardToDiscard: self.! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^'Card' ! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^0.! !

!Card class methodsFor: 'instance creation' stamp: 'JohnParker 7/3/2012 20:03'!
new
	self error: 'Cards can only be created by the Board'! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^self class cardName.! !

!Card methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^self subclassResponsibility! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^self class cost.! !

!Card methodsFor: 'game' stamp: 'JohnParker 7/6/2012 00:48'!
play
	|ret|
	(Board instance currentDeck fromHandToPlay: self class) ifFalse: [self Error: 'You do not have a ' , self cardName , ' in your hand!!'].
	
	[ret := self playCard] on: Error do: [:e | 
		Board instance currentDeck fromPlayToHand: self class.
		e pass.
	].
	^ret.! !

!Card methodsFor: 'private' stamp: 'JohnParker 7/4/2012 01:38'!
playCard
	self SubclassResponsibility ! !

Card subclass: #TreasureCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCard commentStamp: 'JohnParker 7/4/2012 15:05' prior: 0!
A TreasureCard is an abstract class representing a Dominion treasure card.  It adds the value message which returns the amount of treasure the card is worth.!

TreasureCard subclass: #Copper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Copper'! !

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:45'!
cost
	^0! !

!Copper methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:22'!
value
	^1! !

TreasureCard subclass: #Gold
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Gold'! !

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^6! !

!Gold methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
value
	^3! !

TreasureCard subclass: #Silver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Silver'! !

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^3! !

!Silver methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:58'!
value
	^2! !

!TreasureCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:40'!
cardName
	^'Treasure'! !

!TreasureCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:11'!
color
	^Color yellow! !

!TreasureCard methodsFor: 'private' stamp: 'JohnParker 7/4/2012 13:28'!
playCard
	Board instance currentDeck addMoney: self value.! !

!TreasureCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility! !

Card subclass: #VictoryCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!VictoryCard commentStamp: 'JohnParker 7/4/2012 15:06' prior: 0!
A VictoryCard is an abstract class representing a Dominion victory card.  It adds the value message which returns the number of victory points the card is worth. 
It also overrides the playCard message to generate an error since victory cards can not be played.!

VictoryCard subclass: #Curse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cardName 
	^'Curse'! !

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cost
	^0! !

!Curse methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:12'!
color
	^Color purple! !

!Curse methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
value
	^-1! !

VictoryCard subclass: #Duchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Duchy'! !

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^5! !

!Duchy methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
value
	^3! !

VictoryCard subclass: #Estate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cardName 
	^'Estate'! !

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cost
	^2! !

!Estate methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:23'!
value
	^1! !

VictoryCard subclass: #Province
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Provinve'! !

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^8! !

!Province methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
value
	^6! !

!VictoryCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:47'!
cardName 
	^'Victory'! !

!VictoryCard methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/10/2012 22:12'!
color
	^Color green! !

!VictoryCard methodsFor: 'private' stamp: 'JohnParker 7/4/2012 01:48'!
playCard 
	self Error: 'Victory cards can not be played'! !

!VictoryCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility ! !

Object subclass: #Command
	instanceVariableNames: 'deckNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Command commentStamp: 'JohnParker 7/4/2012 14:53' prior: 0!
A Command is an abstract class implementing the command object pattern.  Its reciever is specified by deckNumber.

Instance Variables
	deckNumber:		Number

deckNumber
	- The deck number of the reciever.
!

Command subclass: #CardCommand
	instanceVariableNames: 'cardClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardCommand commentStamp: 'JohnParker 7/4/2012 14:52' prior: 0!
A CardCommand is an abstract class representing a Command that accepts a card class as a parameter

Instance Variables
	cardClass:		Card class

cardClass
	- passed to the reciever as an argument
!

CardCommand subclass: #ActionCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCommand commentStamp: 'JohnParker 7/4/2012 14:50' prior: 0!
An ActionCommand is a command object that sends the playAction: cardClass message to the deck specified by deckNumber!

!ActionCommand methodsFor: 'run' stamp: 'JohnParker 7/4/2012 14:01'!
do
	(Board instance deck: deckNumber) playAction: cardClass! !

CardCommand subclass: #BuyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!BuyCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A BuyCommand is a command object that sends the buy: cardClass message to the deck specified by deckNumber!

!BuyCommand methodsFor: 'run' stamp: 'JohnParker 7/4/2012 00:28'!
do
	(Board instance deck: deckNumber) buy: cardClass! !

!CardCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	^self new deck: aNumber card: aCardClass.! !

!CardCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	deckNumber := aNumber.
	cardClass := aCardClass.
	! !

CardCommand subclass: #TreasureCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A TreasureCommand is a command object that sends the playTreasure: cardClass message to the deck specified by deckNumber!

!TreasureCommand methodsFor: 'run' stamp: 'JohnParker 7/4/2012 12:58'!
do
	(Board instance deck: deckNumber) playTreasure: cardClass! !

!Command class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber
	^self new deck: aNumber.! !

!Command methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:21'!
deck: aNumber
	deckNumber := aNumber.
	! !

!Command methodsFor: 'run' stamp: 'JohnParker 7/4/2012 00:15'!
do
	self SubclassResponsibility.! !

Command subclass: #EndTurnCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!EndTurnCommand commentStamp: 'JohnParker 7/4/2012 15:04' prior: 0!
An EndTurnCommand is a command object that sends the endTurn message to the deck specified by deckNumber
!

!EndTurnCommand methodsFor: 'run' stamp: 'JohnParker 7/4/2012 14:24'!
do
	(Board instance deck: deckNumber) cleanup.! !

Command subclass: #ResolveActionCommand
	instanceVariableNames: 'argVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveActionCommand commentStamp: 'JohnParker 7/10/2012 19:57' prior: 0!
A ResolveActionCommand is a command sent by a player to inform their deck of the way in which they would like to resolve the action in play.  Should be sent in response to a #resolveAction: event from their Deck.

Instance Variables
	argVector:		Array

argVector
	- The list of arguments to be passed.  May vary from card to card.
!

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/6/2012 00:19'!
deck: aNumber argVector: anArgVector
	^self new deck: aNumber argVector: anArgVector! !

!ResolveActionCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/6/2012 00:20'!
deck: aNumber argVector: anArgVector
	deckNumber := aNumber.
	argVector := anArgVector.! !

!ResolveActionCommand methodsFor: 'run' stamp: 'JohnParker 7/6/2012 00:21'!
do
	(Board instance deck: deckNumber) resolveAction: argVector.! !

Object subclass: #Deck
	instanceVariableNames: 'deckNumber draw discard play hand peek reveal actions buys phase money resolving'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Deck commentStamp: 'MichaelOSullivan 7/10/2012 20:39' prior: 0!
A Deck represents all the cards in a Dominion Player's possesion and that player's current turn state.  A card can either be in the draw, discard, hand, play, peek, or reveal areas.  It can only be in one.  A card can only be removed from the deck by being trashed, which is to say it is moved to the trash area of the board.

Instance Variables
	discard:	OrderedCollection
	draw:		OrderedCollection
	hand:		OrderedCollection 
	play:		OrderedCollection
	peek: 		OrderedCollection 
	reveal:		OrderedCollection 
	actions: 	Number
	buys: 		Number
	money: 	Number
	phase: 		Symbol
	resolving:	Boolean
			
discard
	- The players discard pile.

draw
	- The players draw pile.  When this pile is empty the discard pile is shuffled and becomes the new draw pile.

hand
	- The player's current hand.  Drawn as a set of 5 cards during the cleanup phase.

play
	- The play area.  Played cards are placed in this area until the cleanup phase.

peek
	- A list of cards currently being "peeked" at by the player.  Used by cards that require a player to look at cards from their deck and make a decision.

reveal
	- A list of cards currently being "revealed" by the player.  Used by cards that require a player to reveal cards from their deck.

actions
	- The number of actions remaining to be played.

buys
	- The number of buys a player can still make.

money
	- The amount of money the player has this turn due to playing treasure or action cards that give extra money.

phase
	- The phase of the current Turn.  Can be one of:
		#Action 	The player can play actions, treasure, buy cards, or end their turn during this phase
		#Treasure 	The player can play treasure, buy cards, or end their turn during this phase
		#Buy 		The player can buy cards or end their turn during this phase
		#None 		It is not currently the players turn.  Any action will generate an error.

resolving
	- True if the deck is waiting for the player to resolve the last card played.

!

!Deck class methodsFor: 'instance creation' stamp: 'JohnParker 7/5/2012 20:28'!
number: aNumber
	^self new number: aNumber.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
actions
	^actions! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:06'!
addActions: aNumber
	actions := actions + aNumber.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
addBuys: aNumber
	buys := buys + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:43'!
addMoney: aNumber
	money := money + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:22'!
allCards
	|all|
	all := OrderedCollection new.
	all addAll: draw.
	all addAll: discard.
	all addAll: hand.
	all addAll: play.
	^all.! !

!Deck methodsFor: 'attacks' stamp: 'JohnParker 7/4/2012 23:47'!
bureaucrat
	|card|
	card := hand detect: [:each | each class inheritsFrom: VictoryCard] ifNone: [^nil].
	card isNil ifFalse: [
		hand remove: card.
		draw addLast: card.
	]
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:57'!
buy: aCardClass
	(Board instance currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass cost > money) ifTrue: [self error: 'You do not have enough money to purchase a ' , aCardClass cardName].
	(buys = 0) ifTrue: [self cleanup. self error: 'You are out of buys.'].
	resolving ifTrue: [self error: 'You can not buy a card while resolving another card.'].
	
	(aCardClass buy) ifTrue: [
		buys := buys - 1.
		(buys = 0) ifTrue: [self cleanup].
	] ifFalse: [self error: 'Could not buy a ' , aCardClass cardName]
	
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
buys
	^buys! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/5/2012 23:44'!
card: aCardClass from: area1 to: area2
	|card|
	card := area1 detect: [:each | each class = aCardClass] ifNone: [^false].
	area1 remove: card ifAbsent: [^false].
	area2 addLast: card.
	^true.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:57'!
cleanup
	(Board instance currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolving ifTrue: [self error: 'You can not end your turn while resolving a card.'].
	self default.
	Board instance nextDeck.
	! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/5/2012 23:52'!
default
	actions := 1.
	buys := 1.
	money := 0.
	phase := #None.
	resolving := false.
	
	discard addAllLast: play.
	play removeAll.
	
	discard addAllLast: hand.
	hand removeAll.
	
	1 to: 5 do: [:a | self drawCardToHand].! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/5/2012 21:38'!
drawCard
	| card |
	"Reshuffle the discard into the draw if the draw is empty"
	(draw size = 0) ifTrue: [
		draw addAllLast: discard. 
		discard removeAll. 
		draw shuffle.
		self triggerEvent: #playerShuffle withArguments: {deckNumber}.
	].
	"If the draw is still empty, we have no cards to draw so do nothing"
	(draw size = 0) ifFalse: [
		card := draw last.
		draw removeLast.
	].
	self  triggerEvent: #cardDrawn withArguments: {card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 21:31'!
drawCardToDiscard
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		discard add: card.
	].

	Board instance triggerEvent: #cardDiscarded withArguments: {deckNumber. card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 00:02'!
drawCardToHand
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		hand add: card.
	].
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 21:39'!
drawCardToPeek
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		peek add: card.
	].
	self triggerEvent: #cardPeeked withArguments: {card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 20:48'!
drawCardToReveal
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		reveal add: card.
		self triggerEvent: #cardRevealed withArguments: {deckNumber. card class}.
	].
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 23:45'!
fromHandToDiscard: aCardClass
	(self card: aCardClass from: hand to: discard) ifTrue: [
		Board instance triggerEvent: #cardDiscarded withArguments: {deckNumber. aCardClass}.
		^true.
	].
	
	^false.! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 23:46'!
fromHandToPlay: aCardClass
	(self card: aCardClass from: hand to: play) ifTrue: [
		
		^true.
	].
	
	^false.! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 23:47'!
fromPeekToDiscard: aCardClass
	(self card: aCardClass from: peek to: discard) ifTrue: [
		Board instance triggerEvent: #cardDiscarded withArguments: {deckNumber. aCardClass}.
		^true.
	].
	
	^false.! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 23:47'!
fromPeekToHand: aCardClass
	(self card: aCardClass from: peek to: hand) ifTrue: [
		
		^true.
	].! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 23:47'!
fromPlayToHand: aCardClass
	(self card: aCardClass from: play to: hand) ifTrue: [
		
		^true.
	].! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 21:31'!
gainCardToDiscard: aCardClass
	|card|
	card := Board instance takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		discard addLast: card.
		
		Board instance triggerEvent: #cardDiscarded withArguments: {deckNumber. aCardClass}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/5/2012 20:30'!
gainCardToDraw: aCardClass
	|card|
	card := Board instance takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		draw addLast: card.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/6/2012 01:46'!
gainCardToHand: aCardClass
	|card|
	card := Board instance takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		hand addLast: card.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:45'!
hand
	^hand! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 01:34'!
handContains: aCardClass
	^(hand detect: [:each | each class = aCardClass] ifNone: [nil]) isNil not.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/6/2012 00:07'!
handContainsAll: aCardClassVector
	|set|
	set := aCardClassVector asSet.
	^set = (set select: [:testCardClass | 
		(hand count: [:handCard | handCard class = testCardClass]) >=  
			(aCardClassVector count: [:each | each class = testCardClass])
	]).
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:22'!
money
	^money! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 20:29'!
number
	^deckNumber! !

!Deck methodsFor: 'initialize-release' stamp: 'JohnParker 7/5/2012 20:34'!
number: aNumber
	deckNumber := aNumber.
	discard := OrderedCollection new.
	draw := OrderedCollection new.
	hand := OrderedCollection new.
	play := OrderedCollection new.
	peek := OrderedCollection new.
	1 to: 7 do: [:a | self gainCardToDiscard: Copper].
	1 to: 3 do: [:a | self gainCardToDiscard: Estate].
	self default.
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:45'!
phase
	^phase! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:46'!
play
	^play! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/6/2012 02:02'!
playAction: aCardClass
	| card |
	(Board instance currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [self error: aCardClass cardName , ' is not an action card.'].
	(phase = #Treasure) ifTrue: [self error: 'You can not play an action during the treasure phase.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play an action during the buy phase.'].
	resolving ifTrue: [self error: 'You can not play an action while resolving another card.'].
	
	card := hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	actions := actions - 1.
	resolving := card play not.
	
	(actions = 0) ifTrue: [phase := #Treasure].
	
	Board instance triggerEvent: #cardPlayed withArguments: {deckNumber. aCardClass}.
	resolving ifTrue: [self triggerEvent: #resolveAction with: aCardClass].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:58'!
playTreasure: aCardClass
	| card |
	(Board instance currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: TreasureCard) ifFalse: [self error: aCardClass cardName , ' is not a treasure card.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play a treasure during the buy phase.'].
	resolving ifTrue: [self error: 'You can not play a treasure while resolving another card.'].
	
	card := hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	card play.
	phase := #Treasure.
	
	Board instance triggerEvent: #cardPlayed withArguments: {deckNumber. aCardClass}.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/6/2012 01:14'!
resolveAction: argVector
	(Board instance currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolving ifFalse: [self error: 'There is no action to resolve right now.'].
	
	resolving := (play last resolve: argVector) not.
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:30'!
startTurn
	phase := #Action.
	Board instance triggerEvent: #playerTurn withArguments: {deckNumber} .! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/6/2012 01:45'!
trashCardFromHand: aCardClass
	|card|
	card := hand detect: [:each | each class = aCardClass] ifNone: [nil].
	card isNil ifFalse: [
		hand remove: card.
		Board instance trashCard: card deck: aCardClass.
		
		Board instance triggerEvent: #cardTrashed withArguments: {deckNumber. aCardClass}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/4/2012 16:51'!
victoryPoints
	(Board instance gameOver) ifFalse: [self error: 'No peeking!! The game is not over yet!!'].
	^self allCards inject: 0 into: [:points :card |
		(card class inheritsFrom: VictoryCard) ifTrue: [points + card value] ifFalse: [points]
	]! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 14:09'!
z_addCardToHand: aCardClass
	hand add: aCardClass basicNew! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 01:02'!
z_discard
	^discard! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/4/2012 01:02'!
z_draw
	^draw! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/6/2012 00:40'!
z_resolving 
	^resolving ! !

Object subclass: #Player
	instanceVariableNames: 'deckNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Player commentStamp: 'JohnParker 7/10/2012 19:56' prior: 0!
A Player is an abstract class that represents a player.  In this design, a player controls the corresponding deck, sending commands to it in order to play and respond to cards.  A Player also listens for events from Board and Deck which inform it of changes to the game state.  The player may also display the current state of the board if it is a human player.

Instance Variables
	deckNumber:		Number

deckNumber
	- The number of the deck this player controls.
!

!Player methodsFor: 'setup' stamp: 'JohnParker 7/4/2012 12:43'!
deck: aNumber
	deckNumber := aNumber.! !

!Player methodsFor: 'events' stamp: 'JohnParker 7/5/2012 21:18'!
doesNotUnderstand: aMessage
	(Board instance events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	! !

!Player methodsFor: 'events' stamp: 'JohnParker 7/6/2012 02:05'!
resolveAction: aCardClass
	
	(aCardClass = Cellar) ifTrue: [
		| victoryCards |
		victoryCards := (Board instance deck: deckNumber) hand select: [:each | each class inheritsFrom: VictoryCard] thenCollect: [:each | each class].
		Board instance execute: (ResolveActionCommand deck: deckNumber argVector: victoryCards asArray).
	].
	
	(aCardClass = Mine) ifTrue: [
		| treasureCards trashCard gainCard|
		treasureCards := (Board instance deck: deckNumber) hand select: [:each | each class inheritsFrom: TreasureCard] thenCollect: [:each | each class].
		trashCard := treasureCards detectMin: [:each | each cost].
		gainCard := {Silver. Gold} detect: [:each | each cost = (trashCard cost + 3)].
		Board instance execute: (ResolveActionCommand deck: deckNumber argVector: {trashCard. gainCard}).
	].! !
